from . import ugettext_lazy

address_countrycodes = (  # FIXME: Translate from spanish to english
    ('AF', ugettext_lazy('Afganistán')),
    ('AL', ugettext_lazy('Albania')),
    ('DE', ugettext_lazy('Alemania')),
    ('AD', ugettext_lazy('Andorra')),
    ('AO', ugettext_lazy('Angola')),
    ('AI', ugettext_lazy('Anguila')),
    ('AG', ugettext_lazy('Antigua y Barbuda')),
    ('AN', ugettext_lazy('Antillas Holandesas')),
    ('AQ', ugettext_lazy('Antártida')),
    ('SA', ugettext_lazy('Arabia Saudita')),
    ('DZ', ugettext_lazy('Argelia')),
    ('AR', ugettext_lazy('Argentina')),
    ('AM', ugettext_lazy('Armenia')),
    ('AW', ugettext_lazy('Aruba')),
    ('AU', ugettext_lazy('Australia')),
    ('AT', ugettext_lazy('Austria')),
    ('AZ', ugettext_lazy('Azerbaiyán')),
    ('BS', ugettext_lazy('Bahamas')),
    ('BH', ugettext_lazy('Bahráin')),
    ('BD', ugettext_lazy('Bangladesh')),
    ('BB', ugettext_lazy('Barbados')),
    ('BZ', ugettext_lazy('Belice')),
    ('BJ', ugettext_lazy('Benín')),
    ('BM', ugettext_lazy('Bermudas')),
    ('BY', ugettext_lazy('Bielorrusia')),
    ('BO', ugettext_lazy('Bolivia')),
    ('BQ', ugettext_lazy('Bonaire, San Eustaquio y Saba')),
    ('BA', ugettext_lazy('Bosnia y Hercegovina')),
    ('BW', ugettext_lazy('Botsuana')),
    ('BR', ugettext_lazy('Brasil')),
    ('BN', ugettext_lazy('Brunéi')),
    ('BG', ugettext_lazy('Bulgaria')),
    ('BF', ugettext_lazy('Burkina Faso')),
    ('BI', ugettext_lazy('Burundi')),
    ('BT', ugettext_lazy('Bután')),
    ('BE', ugettext_lazy('Bélgica')),
    ('CV', ugettext_lazy('Cabo Verde')),
    ('KH', ugettext_lazy('Camboya')),
    ('CM', ugettext_lazy('Camerún')),
    ('CA', ugettext_lazy('Canadá')),
    ('TD', ugettext_lazy('Chad')),
    ('CZ', ugettext_lazy('Chequia')),
    ('CL', ugettext_lazy('Chile')),
    ('CN', ugettext_lazy('China')),
    ('CY', ugettext_lazy('Chipre')),
    ('VA', ugettext_lazy('Ciudad del Vaticano')),
    ('CO', ugettext_lazy('Colombia')),
    ('KM', ugettext_lazy('Comores')),
    ('CG', ugettext_lazy('Congo')),
    ('KP', ugettext_lazy('Corea del Norte')),
    ('KR', ugettext_lazy('Corea del Sur')),
    ('CR', ugettext_lazy('Costa Rica')),
    ('CI', ugettext_lazy('Costa de Marfil')),
    ('HR', ugettext_lazy('Croacia')),
    ('CU', ugettext_lazy('Cuba')),
    ('CW', ugettext_lazy('Curaçao')),
    ('DK', ugettext_lazy('Dinamarca')),
    ('DM', ugettext_lazy('Dominica')),
    ('EC', ugettext_lazy('Ecuador')),
    ('EG', ugettext_lazy('Egipto')),
    ('SV', ugettext_lazy('El Salvador')),
    ('AE', ugettext_lazy('Emiratos Árabes Unidos')),
    ('ER', ugettext_lazy('Eritrea')),
    ('SK', ugettext_lazy('Eslovaquia')),
    ('SI', ugettext_lazy('Eslovenia')),
    ('ES', ugettext_lazy('España')),
    ('US', ugettext_lazy('Estados Unidos')),
    ('EE', ugettext_lazy('Estonia')),
    ('ET', ugettext_lazy('Etiopía')),
    ('PH', ugettext_lazy('Filipinas')),
    ('FI', ugettext_lazy('Finlandia')),
    ('FJ', ugettext_lazy('Fiyi')),
    ('FR', ugettext_lazy('Francia')),
    ('GA', ugettext_lazy('Gabón')),
    ('GM', ugettext_lazy('Gambia')),
    ('GE', ugettext_lazy('Georgia')),
    ('GH', ugettext_lazy('Ghana')),
    ('GI', ugettext_lazy('Gibraltar')),
    ('GD', ugettext_lazy('Granada')),
    ('GR', ugettext_lazy('Grecia')),
    ('GL', ugettext_lazy('Groenlandia')),
    ('GP', ugettext_lazy('Guadalupe')),
    ('GU', ugettext_lazy('Guam')),
    ('GT', ugettext_lazy('Guatemala')),
    ('GF', ugettext_lazy('Guayana Francesa')),
    ('GG', ugettext_lazy('Guernsey')),
    ('GN', ugettext_lazy('Guinea')),
    ('GQ', ugettext_lazy('Guinea Ecuatorial')),
    ('GW', ugettext_lazy('Guinea-Bissau')),
    ('GY', ugettext_lazy('Guyana')),
    ('HT', ugettext_lazy('Haití')),
    ('NL', ugettext_lazy('Holanda')),
    ('HN', ugettext_lazy('Honduras')),
    ('HK', ugettext_lazy('Hong Kong')),
    ('HU', ugettext_lazy('Hungría')),
    ('IN', ugettext_lazy('India')),
    ('ID', ugettext_lazy('Indonesia')),
    ('IQ', ugettext_lazy('Iraq')),
    ('IE', ugettext_lazy('Irlanda')),
    ('IR', ugettext_lazy('Irán')),
    ('BV', ugettext_lazy('Isla Bouvet')),
    ('CX', ugettext_lazy('Isla Christmas')),
    ('NF', ugettext_lazy('Isla Norfolk')),
    ('IM', ugettext_lazy('Isla de Man')),
    ('IS', ugettext_lazy('Islandia')),
    ('AX', ugettext_lazy('Islas Aland')),
    ('KY', ugettext_lazy('Islas Caimán')),
    ('CC', ugettext_lazy('Islas Cocos')),
    ('CK', ugettext_lazy('Islas Cook')),
    ('FO', ugettext_lazy('Islas Feroe')),
    ('GS', ugettext_lazy('Islas Georgia del Sur y Sandwich del Sur')),
    ('HM', ugettext_lazy('Islas Heard y McDonald')),
    ('FK', ugettext_lazy('Islas Malvinas')),
    ('MP', ugettext_lazy('Islas Marianas del Norte')),
    ('MH', ugettext_lazy('Islas Marshall')),
    ('PN', ugettext_lazy('Islas Pitcairn')),
    ('SB', ugettext_lazy('Islas Salomón')),
    ('TC', ugettext_lazy('Islas Turcas y Caicos')),
    ('VI', ugettext_lazy('Islas Vírgenes Americanas')),
    ('VG', ugettext_lazy('Islas Vírgenes Británicas')),
    ('UM', ugettext_lazy('Islas menores alejadas de los Estados Unidos')),
    ('IL', ugettext_lazy('Israel')),
    ('IT', ugettext_lazy('Italia')),
    ('JM', ugettext_lazy('Jamaica')),
    ('JP', ugettext_lazy('Japón')),
    ('JE', ugettext_lazy('Jersey')),
    ('JO', ugettext_lazy('Jordania')),
    ('KZ', ugettext_lazy('Kazajstán')),
    ('KE', ugettext_lazy('Kenia')),
    ('KG', ugettext_lazy('Kirguizistán')),
    ('KI', ugettext_lazy('Kiribati')),
    ('KW', ugettext_lazy('Kuwait')),
    ('LA', ugettext_lazy('Laos')),
    ('LS', ugettext_lazy('Lesoto')),
    ('LV', ugettext_lazy('Letonia')),
    ('LR', ugettext_lazy('Liberia')),
    ('LY', ugettext_lazy('Libia')),
    ('LI', ugettext_lazy('Liechtenstein')),
    ('LT', ugettext_lazy('Lituania')),
    ('LU', ugettext_lazy('Luxemburgo')),
    ('LB', ugettext_lazy('Líbano')),
    ('MO', ugettext_lazy('Macao')),
    ('MK', ugettext_lazy('Macedonia')),
    ('MG', ugettext_lazy('Madagascar')),
    ('MY', ugettext_lazy('Malasia')),
    ('MW', ugettext_lazy('Malaui')),
    ('MV', ugettext_lazy('Maldivas')),
    ('MT', ugettext_lazy('Malta')),
    ('ML', ugettext_lazy('Malí')),
    ('MA', ugettext_lazy('Marruecos')),
    ('MQ', ugettext_lazy('Martinica')),
    ('MU', ugettext_lazy('Mauricio')),
    ('MR', ugettext_lazy('Mauritania')),
    ('YT', ugettext_lazy('Mayotte')),
    ('FM', ugettext_lazy('Micronesia')),
    ('MD', ugettext_lazy('Moldavia')),
    ('MN', ugettext_lazy('Mongolia')),
    ('ME', ugettext_lazy('Montenegro')),
    ('MS', ugettext_lazy('Montserrat')),
    ('MZ', ugettext_lazy('Mozambique')),
    ('MM', ugettext_lazy('Myanmar')),
    ('MX', ugettext_lazy('México')),
    ('MC', ugettext_lazy('Mónaco')),
    ('NA', ugettext_lazy('Namibia')),
    ('NR', ugettext_lazy('Nauru')),
    ('NP', ugettext_lazy('Nepal')),
    ('NI', ugettext_lazy('Nicaragua')),
    ('NG', ugettext_lazy('Nigeria')),
    ('NU', ugettext_lazy('Niue')),
    ('NO', ugettext_lazy('Noruega')),
    ('NC', ugettext_lazy('Nueva Caledonia')),
    ('NZ', ugettext_lazy('Nueva Zelanda')),
    ('NE', ugettext_lazy('Níger')),
    ('OM', ugettext_lazy('Omán')),
    ('PW', ugettext_lazy('Palaos')),
    ('PS', ugettext_lazy('Palestina')),
    ('PA', ugettext_lazy('Panamá')),
    ('PG', ugettext_lazy('Papúa New Guinea')),
    ('PK', ugettext_lazy('Paquistán')),
    ('PY', ugettext_lazy('Paraguay')),
    ('PE', ugettext_lazy('Perú')),
    ('PF', ugettext_lazy('Polinesia Francesa')),
    ('PL', ugettext_lazy('Polonia')),
    ('PT', ugettext_lazy('Portugal')),
    ('PR', ugettext_lazy('Puerto Rico')),
    ('QA', ugettext_lazy('Qatar')),
    ('GB', ugettext_lazy('Reino Unido')),
    ('CF', ugettext_lazy('República Centroafricana')),
    ('CD', ugettext_lazy('República Democrática del Congo')),
    ('DO', ugettext_lazy('República Dominicana')),
    ('RE', ugettext_lazy('Reunión')),
    ('RW', ugettext_lazy('Ruanda')),
    ('RO', ugettext_lazy('Rumania')),
    ('RU', ugettext_lazy('Rusia')),
    ('EH', ugettext_lazy('Sahara Occidental')),
    ('WS', ugettext_lazy('Samoa')),
    ('AS', ugettext_lazy('Samoa Americana')),
    ('BL', ugettext_lazy('San Bartolomé')),
    ('KN', ugettext_lazy('San Cristóbal y Nieves')),
    ('SM', ugettext_lazy('San Marino')),
    ('MF', ugettext_lazy('San Martín')),
    ('SX', ugettext_lazy('San Martín (región holandesa)')),
    ('PM', ugettext_lazy('San Pedro y Miquelón')),
    ('VC', ugettext_lazy('San Vicente y las Granadinas')),
    ('SH', ugettext_lazy('Santa Helena')),
    ('LC', ugettext_lazy('Santa Lucia')),
    ('ST', ugettext_lazy('Santo Tomé y Príncipe')),
    ('SN', ugettext_lazy('Senegal')),
    ('RS', ugettext_lazy('Serbia')),
    ('SC', ugettext_lazy('Seychelles')),
    ('SL', ugettext_lazy('Sierra Leona')),
    ('SG', ugettext_lazy('Singapur')),
    ('SY', ugettext_lazy('Siria')),
    ('SO', ugettext_lazy('Somalia')),
    ('LK', ugettext_lazy('Sri Lanka')),
    ('SZ', ugettext_lazy('Suazilandia')),
    ('ZA', ugettext_lazy('Sudáfrica')),
    ('SD', ugettext_lazy('Sudán')),
    ('SS', ugettext_lazy('Sudán del Sur')),
    ('SE', ugettext_lazy('Suecia')),
    ('CH', ugettext_lazy('Suiza')),
    ('SR', ugettext_lazy('Surinam')),
    ('SJ', ugettext_lazy('Svalbard y Jan Mayen')),
    ('TH', ugettext_lazy('Tailandia')),
    ('TW', ugettext_lazy('Taiwán')),
    ('TZ', ugettext_lazy('Tanzania')),
    ('TJ', ugettext_lazy('Tayikistán')),
    ('IO', ugettext_lazy('Territorio Británico del Océano Índico')),
    ('TF', ugettext_lazy('Territorios Franceses del Sur')),
    ('TL', ugettext_lazy('Timor Oriental')),
    ('TG', ugettext_lazy('Togo')),
    ('TK', ugettext_lazy('Tokelau')),
    ('TO', ugettext_lazy('Tonga')),
    ('TT', ugettext_lazy('Trinidad y Tobago')),
    ('TM', ugettext_lazy('Turkmenistán')),
    ('TR', ugettext_lazy('Turquía')),
    ('TV', ugettext_lazy('Tuvalu')),
    ('TN', ugettext_lazy('Túnez')),
    ('UA', ugettext_lazy('Ucrania')),
    ('UG', ugettext_lazy('Uganda')),
    ('UY', ugettext_lazy('Uruguay')),
    ('UZ', ugettext_lazy('Uzbekistán')),
    ('VU', ugettext_lazy('Vanuatu')),
    ('VE', ugettext_lazy('Venezuela')),
    ('VN', ugettext_lazy('Vietnam')),
    ('WF', ugettext_lazy('Wallis y Futuna')),
    ('YE', ugettext_lazy('Yemen')),
    ('DJ', ugettext_lazy('Yibuti')),
    ('ZM', ugettext_lazy('Zambia')),
    ('ZW', ugettext_lazy('Zimbabue')),
)

card_brands_unknown = 'unknown'
card_brands_visa = 'visa'
card_brands_mastercard = 'mastercard'
card_brands_carnet = 'carnet'
card_brands_american = 'american_express'
card_brands_american2 = 'american express'
card_brands = (
    (card_brands_unknown, ugettext_lazy('Unknown')),
    (card_brands_visa, ugettext_lazy('Visa')),
    (card_brands_mastercard, ugettext_lazy('Mastercard')),
    (card_brands_carnet, ugettext_lazy('Carnet')),
    (card_brands_american, ugettext_lazy('American Express')),
    (card_brands_american2, ugettext_lazy('American Express')),
)

plan_currency_mxn = 'MXN'
plan_currency_usd = 'USD'
plan_currency = (
    (plan_currency_mxn, ugettext_lazy('MXN')),
    (plan_currency_usd, ugettext_lazy('USD')),
)
plan_status_active = 'active'
plan_status_deleted = 'deleted'
plan_status = (
    (plan_status_active, ugettext_lazy('Active')),
    (plan_status_deleted, ugettext_lazy('Deleted')),
)
plan_statusafter_unpaid = 'unpaid'
plan_statusafter_cancelled = 'cancelled'
plan_statusafter = (
    (plan_statusafter_unpaid, ugettext_lazy('Unpaid')),
    (plan_statusafter_cancelled, ugettext_lazy('Cancelled')),
)
plan_repeatunit_week = 'week'
plan_repeatunit_month = 'month'
plan_repeatunit_year = 'year'
plan_repeatunit = (
    (plan_repeatunit_week, ugettext_lazy('Week')),
    (plan_repeatunit_month, ugettext_lazy('Month')),
    (plan_repeatunit_year, ugettext_lazy('Year')),
)

subscription_status_trial = 'trial'
subscription_status_active = 'active'
subscription_status_pastdue = 'past_due'
subscription_status_unpaid = 'unpaid'
subscription_status_cancelled = 'cancelled'
subscription_status = (
    (subscription_status_trial, ugettext_lazy('Trial')),
    (subscription_status_active, ugettext_lazy('Active')),
    (subscription_status_pastdue, ugettext_lazy('Past Due')),
    (subscription_status_unpaid, ugettext_lazy('Unpaid')),
    (subscription_status_cancelled, ugettext_lazy('Cancelled')),
)

transaction_ttype_fee = 'fee'
transaction_ttype_charge = 'charge'
transaction_ttype_payout = 'payout'
transaction_ttype_transfer = 'transfer'
transaction_ttype = (
    (transaction_ttype_charge, ugettext_lazy('Charge')),
    # TODO: Activate when models are created
    # (transaction_ttype_fee, ugettext_lazy('Fee')),
    # (transaction_ttype_payout, ugettext_lazy('Payout')),
    # (transaction_ttype_transfer, ugettext_lazy('Transfer')),
)
transaction_otype_in = 'in'
transaction_otype_out = 'out'
transaction_otype = (
    (transaction_otype_in, ugettext_lazy('In')),
    (transaction_otype_out, ugettext_lazy('Out')),
)
transaction_method_card = 'card'
transaction_method_bank = 'bank'
transaction_method_customer = 'customer'
transaction_method_store = 'store'
transaction_method_bankaccount = 'bank_account'
transaction_method = (
    (transaction_method_card, ugettext_lazy('Card')),
    # TODO: Activate when models are created
    # (transaction_method_store, ugettext_lazy('Store')),
    # (transaction_method_bank, ugettext_lazy('Bank Account')),
)
transaction_status_completed = 'completed'
transaction_status_inprogress = 'in_progress'
transaction_status_failed = 'failed'
transaction_status = (
    (transaction_status_completed, ugettext_lazy('Completed')),
    (transaction_status_inprogress, ugettext_lazy('In Progress')),
    (transaction_status_failed, ugettext_lazy('Failed')),
)
transaction_currency_mxn = 'MXN'
transaction_currency_usd = 'USD'
transaction_currency = (
    (transaction_currency_mxn, ugettext_lazy('MXN')),
    (transaction_currency_usd, ugettext_lazy('USD')),
)
